@@ -55,7 +55,7 @@ class LearningMachine(object):
                direcs=None,ncpu=40,
                maxiter=1000,
                dt_mlmd=0.1, dt_aimd=0.3,dft_step=5,
-               batch=50,max_batch=10,
+               batch=50,max_batch=100,
                step=20000,md_step=20,MaxMDstep=500,colFrame=10,
                mom_step=100,
                mpopt=1, messages=1,
@@ -70,7 +70,7 @@ class LearningMachine(object):
                rodic=None,
                rmin=0.6,rmax=1.3,angmax=15.0,
                CheckZmat=True,
-               EngTole=0.05,dEtole=0.2,d2Etole=0.05,dEstop=2.0,
+               EngTole=0.05,dEtole=0.2,dEstop=2.0,
                nn=True,vdwnn=False,
                bo_layer=[2,1],mf_layer=[6,1],be_layer=[6,1],
                EnergyFunction=1,MessageFunction=3,
@@ -78,8 +78,7 @@ class LearningMachine(object):
                bore={'others':0.45},
                weight={'others':2.0},
                spv_bm=False,spv_be=False,
-               penalty=10000.0,
-               regularize=True,lambda_pen=0.000001,
+               lambda_bd=10000.0,lambda_reg=0.000001,regularize=True,
                mefac=0.1,
                writelib=5000,
                xcf='VDW',xca='DRSLL',basistype='DZP'):
@@ -123,7 +122,6 @@ class LearningMachine(object):
       self.colFrame     = colFrame
       self.EngTole      = EngTole
       self.dEtole       = dEtole
-      self.d2Etole      = d2Etole
       self.dEstop       = dEstop
       self.MaxMDstep    = MaxMDstep
       self.mpopt        = mpopt
@@ -141,9 +139,9 @@ class LearningMachine(object):
       self.spv_be       = spv_be
       self.mefac        = mefac
       self.weight       = weight
-      self.penalty      = penalty
+      self.lambda_bd    = lambda_bd
       self.regularize   = regularize
-      self.lambda_pen   = lambda_pen
+      self.lambda_reg   = lambda_reg
       self.EnergyFunction = EnergyFunction
       self.MessageFunction= MessageFunction
       self.writelib     = writelib
@@ -334,8 +332,8 @@ class LearningMachine(object):
                                                      bore=self.bore, bom=self.bom,
                                                      spv_be=self.spv_be,spv_bm=self.spv_bm,
                                                      mefac=self.mefac,weight=self.weight,
-                                                     regularize=self.regularize,lambda_pen=self.lambda_pen,
-                                                     penalty=self.penalty)
+                                                     regularize=self.regularize,lambda_reg=self.lambda_reg,
+                                                     lambda_bd=self.lambda_bd)
                 self.load_config()
                 if tIter>=self.resetAcc and tstep>=self.step:
                    if accu>self.accCriteria-self.accTole and loss<self.lossCriteria+self.lossTole:
@@ -435,26 +433,27 @@ class LearningMachine(object):
                   learnWay,Deformed,
                   InitZmat,uz,uzlo,uzhi,optlog,relaxlog):
       with open('learning.log','a') as l:
-         l.write('                  ====================================                  \n')
+         l.write('                  ------------------------------------                  \n')
          l.write('                       Learning iteration:  {:4d}                        \n'.format(i))
-         l.write('                  ====================================                  \n')
+         l.write('                  ------------------------------------                  \n')
          l.write(' \n')
-         l.write('                     I&II  Labeling and Calling DFT ')
+         l.write('                     I&II  Labeling and Calling DFT \n')
          l.write('            Labeled atomic configurations and DFT calculations \n')
+         l.write(' \n')
          l.write(LabelDataLog)
          # for d_,d in enumerate(LabeledData):
          #     if d_%10==0:
          #        l.write('\n')
          #     l.write(' {:3d}'.format(d))
          l.write(' \n')
-         l.write('                           III  Training ')
-         l.write('                          Train the model  \n')
+         l.write('                           III  Training \n')
+         l.write('                     Train the potential model  \n')
          l.write(' \n')
          l.write('Loss of the model:  {:7.4f}  \n'.format(loss))
          l.write('Accuracy of the potential model:  {:7.4f}  \n'.format(accu))
          l.write('\n')
-         l.write('                           IV Exploring \n' %mdsteps)
-         l.write('            Exploring the configuration space by MD simulations \n' %mdsteps)
+         l.write('                           IV Exploring \n')
+         l.write('      Exploring the atomic configuration space by MD simulations \n')
          l.write('\n')
          l.write('MD Steps:  {:d}  \n'.format(mdsteps))
          l.write('Energy from ML and DFT:  {:f}  .vs.  {:f} \n'.format(eirff,edft))
@@ -464,7 +463,7 @@ class LearningMachine(object):
             if Deformed>=1:
                l.write('Molecule structure is deformed, using zmatmix relax\n' )
             elif not uz is None:
-               l.write('relax unreasonable zmatrix variable:\n')
+               l.write('Unreasonable zmatrix variable:\n')
             l.write(relaxlog)
          if not optlog is None:
             l.write(optlog)
@@ -490,7 +489,7 @@ class LearningMachine(object):
          # print(' * files in this dir \n',listdir())
          E,E_,dEmax,d2Emax,ind_ = SinglePointEnergies('md.traj',label=self.label,EngTole=self.EngTole,
                                                  frame=tstep,select=True,
-                                                 dE=self.dEtole,d2E=self.d2Etole,
+                                                 dE=self.dEtole, # d2E=self.d2Etole,
                                                  xcf=self.xcf,xca=self.xca,basistype=self.basistype,
                                                  cpu=self.ncpu)
          eaimd = E
@@ -570,13 +569,12 @@ class LearningMachine(object):
                     'MaxMDstep':self.MaxMDstep,
                     'EngTole':self.EngTole,
                     'dEtole':self.dEtole,
-                    'd2Etole':self.d2Etole,
                     'dEstop':self.dEstop,
                     'beta':self.beta,
                     'mdInc':self.mdInc,
                     'regularize':self.regularize,
-                    'lambda_pen':self.lambda_pen,
-                    'penalty':self.penalty,
+                    'lambda_reg':self.lambda_reg,
+                    'lambda_bd':self.lambda_bd,
                     'rmin':self.rmin,
                     'Tmax':self.Tmax,
                     'T':self.T,
@@ -612,13 +610,12 @@ class LearningMachine(object):
            self.MaxMDstep    = options['MaxMDstep'] 
            self.EngTole      = options['EngTole'] 
            self.dEtole       = options['dEtole'] 
-           self.d2Etole      = options['d2Etole'] 
            self.dEstop       = options['dEstop'] 
            self.beta         = options['beta'] 
            self.mdInc        = options['mdInc'] 
            self.regularize   = options['regularize'] 
-           self.lambda_pen   = options['lambda_pen'] 
-           self.penalty      = options['penalty']
+           self.lambda_reg   = options['lambda_reg'] 
+           self.lambda_bd    = options['lambda_bd']
            self.rmin         = options['rmin'] 
            self.Tmax         = options['Tmax'] 
            self.T            = options['T'] 
@@ -667,7 +664,7 @@ if __name__ == '__main__':
                         mefac=0.005,
                         learnWay=5,
                         weight={'noco':2.0,'others':2.0},
-                        EngTole=0.085,dEtole=0.14,dEstop=1.0,d2Etole=0.06,
+                        EngTole=0.085,dEtole=0.14,dEstop=1.0,
                         rmin=0.7,rmax=1.3,angmax=30.0,
                         bo_layer=[4,1],mf_layer=[6,2],be_layer=[3,1],
                         EnergyFunction=3,MessageFunction=1,
