@@ -5,6 +5,8 @@ email: fengguo@lcu.edu.cn
        gfeng.alan@foxmail.com
 
 This software package is under academic license.
+  2021-05-31:
+     version 1.2.4 All bond share the same neural networks weights and biases.
   2021-05-28:
      version 1.2.3 Regularization algriothm is implemented to overcome the overfit problem.
   2021-05-11:
