@@ -67,8 +67,8 @@ def SinglePointEnergies(traj,label='aimd',xcf='VDW',xca='DRSLL',basistype='DZP',
        ind_.append(id2e)
     ind_.sort()
 
-    LabelDataLog = 'AtomicConfigure   E(ML)   E(DFT)   Diff    dE   d2E  \n'
-    LabelDataLog = '-----------------------------------------------------\n'
+    LabelDataLog = '       AtomicConfigure   E(ML)  E(DFT)   Diff    dE   d2E  \n'
+    LabelDataLog+= '     -------------------------------------------------------\n'
 
     for i in ind_:
         atoms = images[i]
@@ -82,13 +82,13 @@ def SinglePointEnergies(traj,label='aimd',xcf='VDW',xca='DRSLL',basistype='DZP',
         E_.append(e_)
 
         diff_ = abs(e-e_)
-        LabelDataLog += '   {:3d}   {:9.5f}  {:9.5f}  {:6.6f}  {:5.4f}   {:5.4f}\n'.format(i,
+        LabelDataLog += '       {:3d}   {:9.5f}  {:9.5f}  {:6.6f}  {:5.4f}   {:5.4f}\n'.format(i,
                          e_,e,diff_,dE_,d2E_)
         with open('SinglePointEnergies.log','a') as fs:  
              fs.write('%d MLP: %9.5f DFT: %9.5f Diff: %6.6f dE: %5.4f d2E: %5.4f\n' %(i,
                       e_,e,diff_,dE_,d2E_))
             
-        if diff_>EngTole:          # or i==ind_[-1]
+        if diff_>EngTole:  # or i==ind_[-1]
            his.write(atoms=atoms_)
 
     his.close()
