@@ -1147,10 +1147,12 @@ class IRFF_NP(object):
               for i in range(self.natom):
                   mi_ = []
                   for j in range(self.natom):
-                      bd = self.atom_name[i] + '-' + self.atom_name[j]
                       if k_ in ['fe','fesi','fepi','fepp','fsi','fpi','fpp','fv']:
+                         bd = self.atom_name[i] + '-' + self.atom_name[j]
                          if bd not in self.bonds:
                             bd = self.atom_name[j] + '-' + self.atom_name[i]
+                      else:
+                         bd = self.atom_name[i] 
                       key_ = key+'_'+bd
                       if key_ in m:
                          if k in ['bi','bo']:
@@ -1178,10 +1180,12 @@ class IRFF_NP(object):
                   for i in range(self.natom):
                       mi_ = []
                       for j in range(self.natom):
-                          bd = self.atom_name[i] + '-' + self.atom_name[j]
                           if k_ in ['fe','fesi','fepi','fepp','fsi','fpi','fpp','fv']:
+                             bd = self.atom_name[i] + '-' + self.atom_name[j]
                              if bd not in self.bonds:
                                 bd = self.atom_name[j] + '-' + self.atom_name[i]
+                          else:
+                             bd = self.atom_name[i] 
                           key_ = key+'_'+bd
                           if key_ in m:
                              if k == 'b':
@@ -1191,7 +1195,40 @@ class IRFF_NP(object):
                       m_.append(mi_)
                   self.m[key].append(np.array(m_,dtype=np.float32))
 
+  def set_m_univeral(self,m,pres=['fe','fsi','fpi','fpp']):
+      # pres = ['fe','fsi','fpi','fpp']   # general neural network matrix
+      for t in range(1,self.messages+1):
+          pres.append('f'+str(t))
 
+      for k_ in pres:
+          for k in ['wi','bi','wo','bo']:
+              key = k_+k
+              if k in ['bi','bo']:
+                 mi_ = np.expand_dims(np.expand_dims(np.expand_dims(m[key],axis=0),axis=0),axis=0)
+              else:
+                 mi_ = np.expand_dims(np.expand_dims(m[key],axis=0),axis=0)
+              self.m[key] = np.array(mi_,dtype=np.float32)
+
+          for k in ['w','b']:
+              key = k_+k
+              self.m[key] = []
+
+              if k_ in ['fesi','fepi','fepp','fe']:
+                 layer_ = self.be_layer[1]
+              elif k_ in ['fsi','fpi','fpp']:
+                 layer_ = self.bo_layer[1]
+              elif k_ =='fv':
+                 layer_ = self.vdw_layer[1]
+              else:
+                 layer_ = self.mf_layer[1]
+
+              for l in range(layer_):
+                  if k == 'b':
+                     m_ = np.expand_dims(np.expand_dims(np.expand_dims(m[key][l],axis=0),axis=0),axis=0)
+                  else:
+                     m_ = np.expand_dims(np.expand_dims(m[key][l],axis=0),axis=0)
+                  self.m[key].append(np.array(m_,dtype=np.float32))
+                  
   def init_bonds(self):
       self.bonds,self.offd,self.Angs,self.torp,self.Hbs = [],[],[],[],[]
       for key in self.p:
