@@ -32,12 +32,15 @@ class MPNN(ReaxFF):
                dft='ase',atoms=None,
                cons=['val','vale',
                      # 'ovun1','ovun2','ovun3','ovun4',
-                     'ovun5','ovun6','ovun7','ovun8',
-                     'lp1','lp2','lp3',
+                     # 'ovun5','ovun6','ovun7','ovun8',
+                     #'lp1','lp2','lp3',
+                     'cot1','cot2',
+                     'coa1','coa2','coa3','coa4',
                      'pen1','pen2','pen3','pen4',
                      'Depi','Depp','cutoff','acut'], # 'hbtol'
                optmol=True,mefac=0.1,
                opt=None,optword='nocoul',
+               VariablesToOpt=None,
                nanv={'boc1':-2.0},
                batch_size=200,sample='uniform',
                hbshort=6.75,hblong=7.5,
@@ -58,15 +61,18 @@ class MPNN(ReaxFF):
                bo_layer=[4,1],
                mf_layer=[9,2],
                be_layer=[6,1],
-               vdw_layer=[4,1],
+               be_univeral_nn=None,
+               bo_univeral_nn=None,
+               mf_univeral_nn=None,
+               vdw_layer=[4,1],  
                vdwnn=False,
                EnergyFunction=3,
                MessageFunction=1,
                spec=[],
                sort=False,
                pkl=False,
-               penalty=100000.0,
-               lambda_pen=0.000001,
+               lambda_bd=100000.0,
+               lambda_reg=0.0001,
                regularize=False,
                optMethod='ADAM',
                maxstep=60000,
@@ -92,10 +98,15 @@ class MPNN(ReaxFF):
       self.spv_bm           = spv_bm
       self.TwoBodyOnly      = TwoBodyOnly
       self.regularize       = regularize 
-      self.lambda_pen       = lambda_pen
+      self.lambda_reg       = lambda_reg
       self.mf_layer         = mf_layer
       self.be_layer         = be_layer
       self.vdw_layer        = vdw_layer
+
+      self.bo_univeral_nn   = bo_univeral_nn
+      self.be_univeral_nn   = be_univeral_nn
+      self.mf_univeral_nn   = mf_univeral_nn
+
       self.vdwnn            = vdwnn
       if mpopt is None:
          self.mpopt = [True for i in range(messages+3)]
@@ -104,13 +115,13 @@ class MPNN(ReaxFF):
 
       ReaxFF.__init__(self,libfile=libfile,direcs=direcs,
                       dft=dft,atoms=atoms,cons=cons,opt=opt,optword=optword,
-                      optmol=optmol,mefac=mefac,
+                      VariablesToOpt=VariablesToOpt,optmol=optmol,mefac=mefac,
                       nanv=nanv,batch_size=batch_size,sample=sample,
                       hbshort=hbshort,hblong=hblong,vdwcut=vdwcut,
                       ro_scale=ro_scale,
                       clip_op=clip_op,InitCheck=InitCheck,resetDeadNeuron=resetDeadNeuron,nn=nn,
                       bo_layer=bo_layer,spec=spec,sort=sort,pkl=pkl,weight=weight,
-                      bore=bore,penalty=penalty,
+                      bore=bore,lambda_bd=lambda_bd,
                       optMethod=optMethod,maxstep=maxstep,
                       emse=emse,convergence=convergence,lossConvergence=lossConvergence,
                       losFunc=losFunc,conf_vale=conf_vale,
@@ -125,8 +136,20 @@ class MPNN(ReaxFF):
 
   def supervise(self):
       ''' supervised learning term'''
-      l_atol = 0.0
-      self.diffn,self.diffa,self.diffb,self.diffe,self.diffesi = {},{},{},{},{}
+      l_atol  = 0.0
+      pen_w   = 0.0
+      # pen_b = 0.0
+      wb_p    = ['fe','fsi','fpi','fpp']
+      w_n     = ['wi','wo']
+      b_n     = ['bi','b','bo']
+      layer   = {'fe':self.be_layer[1],'fsi':self.bo_layer[1],'fpi':self.bo_layer[1],
+                 'fpp':self.bo_layer[1] }
+      wb_message = []
+      for t in range(1,self.messages+1):
+          wb_message.append('f'+str(t))
+          layer['f'+str(t)] = self.mf_layer[1]
+
+      self.diffn,self.diffa,self.diffb,self.diffe,self.diffesi,self.diffe_= {},{},{},{},{},{}
       for bd in self.bonds: 
           [atomi,atomj] = bd.split('-') 
           if self.nbd[bd]>0:
@@ -142,59 +165,55 @@ class MPNN(ReaxFF):
              
              fao = tf.where(tf.greater(self.rbd[bd],self.rcuta[bd]),1.0,0.0)    ##### r> rcuta that bo = 0.0
              self.diffn[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu(self.bo0[bd]*fao - self.botol*fao))
-             l_atol = tf.add(self.diffn[bd],l_atol)
+             l_atol = tf.add(self.diffn[bd]*self.lambda_bd,l_atol)
 
              if self.spv_bm:
                 fao_= tf.where(tf.less_equal(self.rbd[bd],bm_*self.re[bd]),1.0,0.0) ##### r< 1.2*r_e that bo > botol
                 self.diffa[bd] = tf.reduce_sum(input_tensor=tf.nn.relu((self.botol-self.bo0[bd])*fao_))
-                l_atol = tf.add(self.diffa[bd],l_atol)
+                l_atol = tf.add(self.diffa[bd]*self.lambda_bd,l_atol)
 
              #  bop should be zero if r>rcut
              fbo = tf.where(tf.less_equal(self.rbd[bd],self.rc_bo[bd]),0.0,1.0) ##### r> rc_bo that bo < botol
              self.diffb[bd]  = tf.reduce_sum(self.bop[bd]*fbo)
-             l_atol  = tf.add(self.diffb[bd],l_atol)
+             l_atol  = tf.add(self.diffb[bd]*self.lambda_bd,l_atol)
              
              if self.spv_be:
                 fe  = tf.where(tf.less_equal(self.rbd[bd],self.re[bd]),1.0,0.0) ##### r< r_e that bo > bore_
                 self.diffe[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu((bore_-self.bo0[bd])*fe))
-                l_atol  = tf.add(self.diffe[bd],l_atol)
+                l_atol  = tf.add(self.diffe[bd]*self.lambda_bd,l_atol)
             
-                fe  = tf.where(tf.less_equal(self.rbd[bd],self.re[bd]),1.0,0.0) ##### r< r_e that bo > bore_
-                self.diffe[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu((0.2*self.p['Desi_'+bd]+self.EBD[bd])*fe))
-                l_atol  = tf.add(self.diffe[bd],l_atol)
+                self.diffe_[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu((bore_-self.esi[bd])*fe))
+                l_atol  = tf.add(self.diffe_[bd]*self.lambda_bd,l_atol)          ## penalty iterm for bond energy
 
              if self.EnergyFunction == 1:
-                fe  = tf.where(tf.less_equal(self.bo0[bd],self.botol),1.0,0.0) ##### bo <= 0.0 that e = 0.0
-                self.diffesi[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu((-self.EBD[bd])*fe))
-                l_atol  = tf.add(self.diffesi[bd],l_atol)
+                fe  = tf.where(tf.less(self.bo0[bd],self.botol),1.0,0.0) ##### bo <= 0.0 that e = 0.0
+                self.diffesi[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu((self.esi[bd])*fe))
+                l_atol  = tf.add(self.diffesi[bd]*self.lambda_bd,l_atol)
+             
+             pen_rcut = tf.nn.relu(self.rc_bo[bd]-self.rcut[bd])
+             l_atol= tf.add(pen_rcut*self.lambda_bd,l_atol)
 
              # penalize term for regularization of the neural networs
-             if self.regularize:                                               # regularize
-                pen_bo_w  = tf.reduce_sum(tf.square(self.m['fsiwi_'+bd]))
-                pen_bo_w += tf.reduce_sum(tf.square(self.m['fpiwi_'+bd]))
-                pen_bo_w += tf.reduce_sum(tf.square(self.m['fppwi_'+bd]))
-                for l in range(self.bo_layer[1]):                                               
-                    pen_bo_w += tf.reduce_sum(tf.square(self.m['fsiw_'+bd][l]))
-                    pen_bo_w += tf.reduce_sum(tf.square(self.m['fpiw_'+bd][l]))
-                    pen_bo_w += tf.reduce_sum(tf.square(self.m['fppw_'+bd][l]))
-                pen_bo_w += tf.reduce_sum(tf.square(self.m['fsiwo_'+bd])) 
-                pen_bo_w += tf.reduce_sum(tf.square(self.m['fpiwo_'+bd])) 
-                pen_bo_w += tf.reduce_sum(tf.square(self.m['fppwo_'+bd])) 
+             if self.regularize:                                         # regularize
+                for k in wb_p:
+                    for k_ in w_n:
+                        key     = k + k_ + '_' + bd
+                        pen_w  += tf.reduce_sum(tf.square(self.m[key]))
 
-                for t in range(1,self.messages+1):
-                    pen_bo_w  += tf.reduce_sum(tf.square(self.m['f'+str(t)+'wi_'+bd]))
-                    for l in range(self.mf_layer[1]):                                               
-                        pen_bo_w += tf.reduce_sum(tf.square(self.m['f'+str(t)+'w_'+bd][l]))
-                    pen_bo_w += tf.reduce_sum(tf.square(self.m['f'+str(t)+'wo_'+bd])) 
+                    for l in range(layer[k]):                                               
+                        pen_w += tf.reduce_sum(tf.square(self.m[k+'w_'+bd][l]))
 
-                pen_bo_w  = tf.reduce_sum(tf.square(self.m['fewi_'+bd]))
-                for l in range(self.be_layer[1]):                                               
-                    pen_bo_w += tf.reduce_sum(tf.square(self.m['few_'+bd][l]))
-                pen_bo_w += tf.reduce_sum(tf.square(self.m['fewo_'+bd])) 
-                l_atol= tf.add(self.lambda_pen*pen_bo_w,l_atol)
+      if self.regularize:                                               # regularize
+         for sp in self.spec:
+             for k in wb_message:
+                 for k_ in w_n:
+                     key     = k + k_ + '_' + sp
+                     pen_w  += tf.reduce_sum(tf.square(self.m[key]))
+                 for l in range(layer[k]):                                               
+                     pen_w += tf.reduce_sum(tf.square(self.m[k+'w_'+sp][l]))
 
-             l_atol= tf.add(tf.nn.relu(self.rc_bo[bd]-self.rcut[bd]),l_atol)
-      return l_atol*self.penalty
+      l_atol = tf.add(self.lambda_reg*pen_w,l_atol)
+      return l_atol
 
   def get_loss(self):
       self.Loss = 0.0
@@ -323,12 +342,12 @@ class MPNN(ReaxFF):
       X   = tf.stack(x_,axis=1)        # Dimention: (nbatch,4)
                                        #        Wi:  (4,8) 
       o   =  []                        #        Wh:  (8,8)
-      o.append(tf.sigmoid(tf.matmul(X,self.m[pre+'wi_'+bd],name='bop_input')+self.m[pre+'bi_'+bd]))   # input layer
+      o.append(tf.sigmoid(tf.matmul(X,self.m[pre+'wi_'+bd],name='input')+self.m[pre+'bi_'+bd]))   # input layer
 
       for l in range(layer):                                                   # hidden layer      
-          o.append(tf.sigmoid(tf.matmul(o[-1],self.m[pre+'w_'+bd][l],name='bop_hide')+self.m[pre+'b_'+bd][l]))
+          o.append(tf.sigmoid(tf.matmul(o[-1],self.m[pre+'w_'+bd][l],name='hide')+self.m[pre+'b_'+bd][l]))
 
-      o_ = tf.sigmoid(tf.matmul(o[-1],self.m[pre+'wo_'+bd],name='bop_output') + self.m[pre+'bo_'+bd])  # output layer
+      o_ = tf.sigmoid(tf.matmul(o[-1],self.m[pre+'wo_'+bd],name='output') + self.m[pre+'bo_'+bd])  # output layer
       out= tf.reshape(o_,[nbd,self.batch])
       return out
 
@@ -401,16 +420,16 @@ class MPNN(ReaxFF):
       bdr     = b[1]+'-'+b[0]
       flabel  = 'f'+str(t)
       if self.MessageFunction==1:
-         Fi    = self.f_nn(flabel,bd, self.nbd[bd],[Dbi,Dbj,h],layer=self.mf_layer[1])
-         Fj    = self.f_nn(flabel,bdr,self.nbd[bd],[Dbj,Dbi,h],layer=self.mf_layer[1])
+         Fi    = self.f_nn(flabel,b[0],self.nbd[bd],[Dbi,Dbj,h],layer=self.mf_layer[1])
+         Fj    = self.f_nn(flabel,b[1],self.nbd[bd],[Dbj,Dbi,h],layer=self.mf_layer[1])
          F     = 2.0*Fi*Fj
        
          bosi  = self.Hsi[t-1][bd]*F
          bopi  = self.Hpi[t-1][bd]*F
          bopp  = self.Hpp[t-1][bd]*F
       elif self.MessageFunction==2:
-         Fi    = self.fmessage(flabel,bd, self.nbd[bd],[Dbi,Dbj,h],layer=self.mf_layer[1])
-         Fj    = self.fmessage(flabel,bdr,self.nbd[bd],[Dbj,Dbi,h],layer=self.mf_layer[1])
+         Fi    = self.fmessage(flabel,b[0],self.nbd[bd],[Dbi,Dbj,h],layer=self.mf_layer[1])
+         Fj    = self.fmessage(flabel,b[1],self.nbd[bd],[Dbj,Dbi,h],layer=self.mf_layer[1])
          F     = Fi*Fj
          Fsi,Fpi,Fpp = tf.unstack(F,axis=2)
        
@@ -418,8 +437,8 @@ class MPNN(ReaxFF):
          bopi  = self.Hpi[t-1][bd]*Fpi
          bopp  = self.Hpp[t-1][bd]*Fpp
       elif self.MessageFunction==3:
-         Fi    = self.f_nn(flabel,bd, self.nbd[bd],[Dbi,Dbj,h],layer=self.mf_layer[1])
-         Fj    = self.f_nn(flabel,bdr,self.nbd[bd],[Dbj,Dbi,h],layer=self.mf_layer[1])
+         Fi    = self.f_nn(flabel,b[0],self.nbd[bd],[Dbi,Dbj,h],layer=self.mf_layer[1])
+         Fj    = self.f_nn(flabel,b[1],self.nbd[bd],[Dbj,Dbi,h],layer=self.mf_layer[1])
          F     = Fi*Fj
        
          bosi  = self.Hsi[t-1][bd]*F
@@ -541,12 +560,11 @@ class MPNN(ReaxFF):
                                   layer=self.be_layer[1])
          self.EBD[bd] = -self.p['Desi_'+bd]*self.esi[bd] # *self.bo0[bd]
       elif self.EnergyFunction==2:
-         Fi = self.f_nn('fe',bd, self.nbd[bd],[Dbi,Dbj,self.bosi[bd]],layer=self.be_layer[1])
-         Fj = self.f_nn('fe',bdr,self.nbd[bd],[Dbj,Dbi,self.bosi[bd]],layer=self.be_layer[1])
-         F  = Fi*Fj
-    
-         self.esi[bd]  = self.bosi[bd]*F       
-         self.sieng[bd]= tf.multiply(self.p['Desi_'+bd],self.esi[bd])
+         # Fi = self.f_nn('fe',bd, self.nbd[bd],[Dbi,Dbj,self.bosi[bd]],layer=self.be_layer[1])
+         # Fj = self.f_nn('fe',bdr,self.nbd[bd],[Dbj,Dbi,self.bosi[bd]],layer=self.be_layer[1])
+         # F  = Fi*Fj
+         # self.esi[bd]  = self.bosi[bd]*F       
+         self.sieng[bd]= tf.multiply(self.p['Desi_'+bd],self.bosi[bd])
          self.pieng[bd]= tf.multiply(self.p['Depi_'+bd],self.bopi[bd])
          self.ppeng[bd]= tf.multiply(self.p['Depp_'+bd],self.bopp[bd]) 
 
@@ -555,43 +573,46 @@ class MPNN(ReaxFF):
   def set_m(self):
       ''' set variable for neural networks '''
       self.m = {}
-      bond   = []
+      bond   = []               # make sure the m matrix is unique 
       for si in self.spec:
           for sj in self.spec:
               bd = si + '-' + sj
               if bd not in bond:
                  bond.append(bd)
+      self.get_univeral_nn()
 
       reuse_m = True if self.bo_layer==self.bo_layer_ else False
-      self.set_wb(pref='fsi',reuse_m=reuse_m,nin=1,nout=1,layer=self.bo_layer,
-                  vlist=self.bonds,nnopt=self.mpopt[0],bias=2.0)
-      self.set_wb(pref='fpi',reuse_m=reuse_m,nin=1,nout=1,layer=self.bo_layer,
-                  vlist=self.bonds,nnopt=self.mpopt[0],bias=2.0)
-      self.set_wb(pref='fpp',reuse_m=reuse_m,nin=1,nout=1,layer=self.bo_layer,
-                  vlist=self.bonds,nnopt=self.mpopt[0],bias=2.0)
+      for p_ in ['fsi','fpi','fpp']:
+          if not self.bo_univeral_nn is None:
+             self.set_univeral_wb(pref=p_,bd=self.bo_univeral_nn[0],reuse_m=reuse_m,
+                                    nin=1,nout=1,layer=self.bo_layer,
+                                    nnopt=self.mpopt[0],bias=2.0)
+          self.set_wb(pref=p_,reuse_m=reuse_m,nin=1,nout=1,layer=self.bo_layer,
+                      vlist=self.bonds,nnopt=self.mpopt[0],bias=2.0)
  
+      ############ set weight and bias for message neural network ###################
       reuse_m = True if self.mf_layer==self.mf_layer_ else False
       for t in range(1,self.messages+1):
           b = 0.881373587 if t>1 else -0.867
           nout_ = 1 if self.MessageFunction==1 or self.MessageFunction==3 else 3
-          self.set_wb(pref='f'+str(t),reuse_m=reuse_m,nin=3,nout=nout_,layer=self.mf_layer,
-                      vlist=bond,nnopt=self.mpopt[t],bias=b)
+          if not self.mf_univeral_nn is None:
+             self.set_univeral_wb(pref='f'+str(t),bd=self.mf_univeral_nn[0],reuse_m=reuse_m,
+                                    nin=3,nout=nout_,layer=self.mf_layer,
+                                    nnopt=self.mpopt[t],bias=b)
+          self.set_message_wb(pref='f'+str(t),reuse_m=reuse_m,nin=3,nout=nout_,
+                      vlist=bond,layer=self.mf_layer,nnopt=self.mpopt[t],bias=b) 
 
+      ############ set weight and bias for energy neural network ###################
       if self.EnergyFunction==self.EnergyFunction_ and self.be_layer==self.be_layer_:
          reuse_m = True  
       else:
          reuse_m = False 
-
-      # if self.EnergyFunction==1:
-      #    self.set_wb(pref='fesi',reuse_m=reuse_m,nin=1,nout=1,layer=self.be_layer,
-      #                vlist=self.bonds,nnopt=self.mpopt[-2],bias=2.0)
-      #    self.set_wb(pref='fepi',reuse_m=reuse_m,nin=1,nout=1,layer=self.be_layer,
-      #                vlist=self.bonds,nnopt=self.mpopt[-2],bias=2.0)
-      #    self.set_wb(pref='fepp',reuse_m=reuse_m,nin=1,nout=1,layer=self.be_layer,
-      #                vlist=self.bonds,nnopt=self.mpopt[-2],bias=2.0)
-      # elif self.EnergyFunction>=2:
+      if not self.be_univeral_nn is None:
+         self.set_univeral_wb(pref='fe',bd=self.be_univeral_nn[0],reuse_m=reuse_m,
+                                nin=3,nout=1,layer=self.be_layer,
+                                nnopt=self.mpopt[t+1],bias=2.0)
       self.set_wb(pref='fe',reuse_m=reuse_m,nin=3,nout=1,layer=self.be_layer,
-                  vlist=self.bonds,nnopt=self.mpopt[-2],bias=2.0)
+                  vlist=self.bonds,nnopt=self.mpopt[t+1],bias=2.0)
 
       if self.vdwnn:
          reuse_m = True if self.vdw_layer==self.vdw_layer_ else False
@@ -603,40 +624,47 @@ class MPNN(ReaxFF):
       if self.m_ is None:
          self.m_ = {}
       for bd in vlist:
-          if pref+'wi_'+bd in self.m_ and reuse_m:                   # input layer
-              if nnopt:
-                 # print(self.m_['fwi_'+bd])
-                 self.m[pref+'wi_'+bd] = tf.Variable(self.m_[pref+'wi_'+bd],name=pref+'wi_'+bd)
-                 self.m[pref+'bi_'+bd] = tf.Variable(self.m_[pref+'bi_'+bd],name=pref+'bi_'+bd)
-              else:
-                 self.m[pref+'wi_'+bd] = tf.constant(self.m_[pref+'wi_'+bd],name=pref+'wi_'+bd)
-                 self.m[pref+'bi_'+bd] = tf.constant(self.m_[pref+'bi_'+bd],name=pref+'bi_'+bd)
+          if pref+'_'+bd in self.univeral_nn:
+             self.m[pref+'wi_'+bd] = self.m[pref+'wi']
+             self.m[pref+'bi_'+bd] = self.m[pref+'bi']
+          elif pref+'wi_'+bd in self.m_ and reuse_m:                   # input layer
+             if nnopt:
+                self.m[pref+'wi_'+bd] = tf.Variable(self.m_[pref+'wi_'+bd],name=pref+'wi_'+bd)
+                self.m[pref+'bi_'+bd] = tf.Variable(self.m_[pref+'bi_'+bd],name=pref+'bi_'+bd)
+             else:
+                self.m[pref+'wi_'+bd] = tf.constant(self.m_[pref+'wi_'+bd],name=pref+'wi_'+bd)
+                self.m[pref+'bi_'+bd] = tf.constant(self.m_[pref+'bi_'+bd],name=pref+'bi_'+bd)
           else:
-              self.m[pref+'wi_'+bd] = tf.Variable(tf.random.normal([nin,layer[0]],stddev=0.1),name=pref+'wi_'+bd)   
-              self.m[pref+'bi_'+bd] = tf.Variable(tf.random.normal([layer[0]],stddev=0.1),
+             self.m[pref+'wi_'+bd] = tf.Variable(tf.random.normal([nin,layer[0]],stddev=0.1),name=pref+'wi_'+bd)   
+             self.m[pref+'bi_'+bd] = tf.Variable(tf.random.normal([layer[0]],stddev=0.1),
                                                   name=pref+'bi_'+bd)  
 
           self.m[pref+'w_'+bd] = []                                    # hidden layer
           self.m[pref+'b_'+bd] = []
-          if pref+'w_'+bd in self.m_ and reuse_m:     
-              if nnopt:                            
-                 for i in range(layer[1]):   
-                     self.m[pref+'w_'+bd].append(tf.Variable(self.m_[pref+'w_'+bd][i],name=pref+'wh'+str(i)+'_'+bd )) 
-                     self.m[pref+'b_'+bd].append(tf.Variable(self.m_[pref+'b_'+bd][i],name=pref+'bh'+str(i)+'_'+bd )) 
-              else:
-                 for i in range(layer[1]):   
-                     self.m[pref+'w_'+bd].append(tf.constant(self.m_[pref+'w_'+bd][i],name=pref+'wh'+str(i)+'_'+bd )) 
-                     self.m[pref+'b_'+bd].append(tf.constant(self.m_[pref+'b_'+bd][i],name=pref+'bh'+str(i)+'_'+bd )) 
+          if pref+'_'+bd in self.univeral_nn:
+             for i in range(layer[1]):  
+                 self.m[pref+'w_'+bd] = self.m[pref+'w']
+                 self.m[pref+'b_'+bd] = self.m[pref+'b']
+          elif pref+'w_'+bd in self.m_ and reuse_m:     
+             if nnopt:                            
+                for i in range(layer[1]):   
+                    self.m[pref+'w_'+bd].append(tf.Variable(self.m_[pref+'w_'+bd][i],name=pref+'wh'+str(i)+'_'+bd )) 
+                    self.m[pref+'b_'+bd].append(tf.Variable(self.m_[pref+'b_'+bd][i],name=pref+'bh'+str(i)+'_'+bd )) 
+             else:
+                for i in range(layer[1]):   
+                    self.m[pref+'w_'+bd].append(tf.constant(self.m_[pref+'w_'+bd][i],name=pref+'wh'+str(i)+'_'+bd )) 
+                    self.m[pref+'b_'+bd].append(tf.constant(self.m_[pref+'b_'+bd][i],name=pref+'bh'+str(i)+'_'+bd )) 
           else:
-              for i in range(layer[1]):   
-                  self.m[pref+'w_'+bd].append(tf.Variable(tf.random.normal([layer[0],layer[0]], 
-                                                           stddev=0.1),name=pref+'wh'+str(i)+'_'+bd)) 
-                  self.m[pref+'b_'+bd].append(tf.Variable(tf.random.normal([layer[0]], 
-                                                           stddev=0.1),name=pref+'bh'+str(i)+'_'+bd )) 
+             for i in range(layer[1]):   
+                 self.m[pref+'w_'+bd].append(tf.Variable(tf.random.normal([layer[0],layer[0]], 
+                                                         stddev=0.1),name=pref+'wh'+str(i)+'_'+bd)) 
+                 self.m[pref+'b_'+bd].append(tf.Variable(tf.random.normal([layer[0]], 
+                                                         stddev=0.1),name=pref+'bh'+str(i)+'_'+bd )) 
           
-          reuse_ = reuse_m if self.MessageFunction == self.MessageFunction_ else False
-
-          if pref+'wo_'+bd in self.m_ and reuse_:                                 # output layer
+          if pref+'_'+bd in self.univeral_nn:
+             self.m[pref+'wo_'+bd] = self.m[pref+'wo']
+             self.m[pref+'bo_'+bd] = self.m[pref+'bo']
+          elif pref+'wo_'+bd in self.m_ and reuse_m:          # output layer
               if nnopt:       
                  self.m[pref+'wo_'+bd] = tf.Variable(self.m_[pref+'wo_'+bd],name=pref+'wo_'+bd)
                  self.m[pref+'bo_'+bd] = tf.Variable(self.m_[pref+'bo_'+bd],name=pref+'bo_'+bd)
@@ -647,6 +675,152 @@ class MPNN(ReaxFF):
               self.m[pref+'wo_'+bd] = tf.Variable(tf.random.normal([layer[0],nout],stddev=0.0001), name=pref+'wo_'+bd)   
               self.m[pref+'bo_'+bd] = tf.Variable(tf.random.normal([nout], stddev=0.0001)+bias,
                                                   name=pref+'bo_'+bd)
+  
+  def set_message_wb(self,pref='f',reuse_m=True,nnopt=True,nin=8,nout=3,layer=[8,9],vlist=None,bias=0.0):
+      ''' set matix varibles '''
+      if self.m_ is None:
+         self.m_ = {}
+      for sp in self.spec:
+          self.m[pref+'w_'+sp] = []                                    
+          self.m[pref+'b_'+sp] = []
+          # self.m[pref+'w_'+sp+'-'+sp]  = []                                    
+          # self.m[pref+'b_'+sp+'-'+sp]  = []
+          if pref+'_'+sp in self.univeral_nn:
+             self.m[pref+'wi_'+sp] = self.m[pref+'wi']
+             self.m[pref+'bi_'+sp] = self.m[pref+'bi']
+             self.m[pref+'wo_'+sp] = self.m[pref+'wo']
+             self.m[pref+'bo_'+sp] = self.m[pref+'bo']
+             self.m[pref+'w_'+sp]  = self.m[pref+'w'] 
+             self.m[pref+'b_'+sp]  = self.m[pref+'b'] 
+          elif pref+'wi_'+sp in self.m_ and reuse_m:
+             if nnopt:
+                self.m[pref+'wi_'+sp] = tf.Variable(self.m_[pref+'wi_'+sp],name=pref+'wi_'+sp)
+                self.m[pref+'bi_'+sp] = tf.Variable(self.m_[pref+'bi_'+sp],name=pref+'bi_'+sp)
+                self.m[pref+'wo_'+sp] = tf.Variable(self.m_[pref+'wo_'+sp],name=pref+'wo_'+sp)
+                self.m[pref+'bo_'+sp] = tf.Variable(self.m_[pref+'bo_'+sp],name=pref+'bo_'+sp)
+                for i in range(layer[1]):   
+                    self.m[pref+'w_'+sp].append(tf.Variable(self.m_[pref+'w_'+sp][i],name=pref+'wh'+str(i)+'_'+sp )) 
+                    self.m[pref+'b_'+sp].append(tf.Variable(self.m_[pref+'b_'+sp][i],name=pref+'bh'+str(i)+'_'+sp )) 
+             else:
+                self.m[pref+'wi_'+sp] = tf.constant(self.m_[pref+'wi_'+sp],name=pref+'wi_'+sp)
+                self.m[pref+'bi_'+sp] = tf.constant(self.m_[pref+'bi_'+sp],name=pref+'bi_'+sp)
+                self.m[pref+'wo_'+sp] = tf.constant(self.m_[pref+'wo_'+sp],name=pref+'wo_'+sp)
+                self.m[pref+'bo_'+sp] = tf.constant(self.m_[pref+'bo_'+sp],name=pref+'bo_'+sp)
+                for i in range(layer[1]):   
+                    self.m[pref+'w_'+sp].append(tf.constant(self.m_[pref+'w_'+sp][i],name=pref+'wh'+str(i)+'_'+sp )) 
+                    self.m[pref+'b_'+sp].append(tf.constant(self.m_[pref+'b_'+sp][i],name=pref+'bh'+str(i)+'_'+sp )) 
+          else:
+             self.m[pref+'wi_'+sp] = tf.Variable(tf.random.normal([nin,layer[0]],stddev=0.1),name=pref+'wi_'+sp)   
+             self.m[pref+'bi_'+sp] = tf.Variable(tf.random.normal([layer[0]],stddev=0.1),name=pref+'bi_'+sp)  
+             self.m[pref+'wo_'+sp] = tf.Variable(tf.random.normal([nin,layer[0]],stddev=0.1),name=pref+'wo_'+sp)   
+             self.m[pref+'bo_'+sp] = tf.Variable(tf.random.normal([layer[0]],stddev=0.1),name=pref+'bo_'+sp)  
+             for i in range(layer[1]):   
+                 self.m[pref+'w_'+sp].append(tf.Variable(tf.random.normal([layer[0],layer[0]], 
+                                                         stddev=0.1),name=pref+'wh'+str(i)+'_'+bd)) 
+                 self.m[pref+'b_'+sp].append(tf.Variable(tf.random.normal([layer[0]], 
+                                                         stddev=0.1),name=pref+'bh'+str(i)+'_'+bd )) 
+
+          # self.m[pref+'wi_'+sp+'-'+sp] = self.m[pref+'wi_'+sp]
+          # self.m[pref+'bi_'+sp+'-'+sp] = self.m[pref+'bi_'+sp]
+          # self.m[pref+'wo_'+sp+'-'+sp] = self.m[pref+'wo_'+sp]
+          # self.m[pref+'bo_'+sp+'-'+sp] = self.m[pref+'bo_'+sp]
+          # self.m[pref+'w_'+sp+'-'+sp]  = self.m[pref+'w_'+sp]
+          # self.m[pref+'b_'+sp+'-'+sp]  = self.m[pref+'b_'+sp]
+
+      # for bd in vlist:
+      #     b = bd.split('-')
+      #     sp= b[0]
+      #     if b[0]!=b[1]:                   # input layer
+      #        self.m[pref+'wi_'+bd] = self.m[pref+'wi_'+sp]
+      #        self.m[pref+'bi_'+bd] = self.m[pref+'bi_'+sp]
+      #        self.m[pref+'wo_'+bd] = self.m[pref+'wo_'+sp]
+      #        self.m[pref+'bo_'+bd] = self.m[pref+'bo_'+sp]
+      #        self.m[pref+'w_'+bd]  = self.m[pref+'w_'+sp]
+      #        self.m[pref+'b_'+bd]  = self.m[pref+'b_'+sp]
+
+
+  def set_univeral_wb(self,pref='f',bd='C-C',reuse_m=True,nnopt=True,nin=8,nout=3,
+                         layer=[8,9],bias=0.0):
+      ''' set universial matix varibles '''
+      if self.m_ is None:
+         self.m_ = {}
+
+      self.m[pref+'w'] = []                                    # hidden layer
+      self.m[pref+'b'] = []
+
+      if pref+'wi' in self.m_:
+          bd_ = ''
+      else:
+          bd_ = '_' + bd
+      
+      if reuse_m:                   # input layer
+         if nnopt:
+            self.m[pref+'wi'] = tf.Variable(self.m_[pref+'wi'+bd_],name=pref+'wi')
+            self.m[pref+'bi'] = tf.Variable(self.m_[pref+'bi'+bd_],name=pref+'bi')
+            self.m[pref+'wo'] = tf.Variable(self.m_[pref+'wo'+bd_],name=pref+'wo')
+            self.m[pref+'bo'] = tf.Variable(self.m_[pref+'bo'+bd_],name=pref+'bo')
+            for i in range(layer[1]):   
+                self.m[pref+'w'].append(tf.Variable(self.m_[pref+'w'+bd_][i],
+                                                    name=pref+'wh'+str(i))) 
+                self.m[pref+'b'].append(tf.Variable(self.m_[pref+'b'+bd_][i],
+                                                    name=pref+'bh'+str(i))) 
+         else:
+            self.m[pref+'wi'] = tf.constant(self.m_[pref+'wi'+bd_],name=pref+'wi')
+            self.m[pref+'bi'] = tf.constant(self.m_[pref+'bi'+bd_],name=pref+'bi')
+            self.m[pref+'wo'] = tf.constant(self.m_[pref+'wo'+bd_],name=pref+'wo')
+            self.m[pref+'bo'] = tf.constant(self.m_[pref+'bo'+bd_],name=pref+'bo')
+            for i in range(layer[1]):   
+                self.m[pref+'w'].append(tf.constant(self.m_[pref+'w'+bd_][i],
+                                                    name=pref+'wh'+str(i))) 
+                self.m[pref+'b'].append(tf.constant(self.m_[pref+'b'+bd_][i],
+                                                    name=pref+'bh'+str(i))) 
+      else:
+         self.m[pref+'wi'] = tf.Variable(tf.random.normal([nin,layer[0]],stddev=0.1),name=pref+'wi')   
+         self.m[pref+'bi'] = tf.Variable(tf.random.normal([layer[0]],stddev=0.1),name=pref+'bi')  
+         self.m[pref+'wo'] = tf.Variable(tf.random.normal([layer[0],nout],stddev=0.0001),name=pref+'wo')   
+         self.m[pref+'bo'] = tf.Variable(tf.random.normal([nout], stddev=0.0001)+bias,name=pref+'bo')
+         for i in range(layer[1]):   
+             self.m[pref+'w'].append(tf.Variable(tf.random.normal([layer[0],layer[0]], 
+                                                 stddev=0.1),name=pref+'wh'+str(i))) 
+             self.m[pref+'b'].append(tf.Variable(tf.random.normal([layer[0]], 
+                                                 stddev=0.1),name=pref+'bh'+str(i))) 
+
+  def get_univeral_nn(self):
+      self.univeral_nn = []
+      if not self.bo_univeral_nn is None:
+         if self.bo_univeral_nn=='all':
+            univeral_bonds = self.bonds
+         else:
+            univeral_bonds = self.bo_univeral_nn
+         for bd in univeral_bonds:
+             b = bd.split('-')
+             bdr = b[1] + '-' + b[0]
+             self.univeral_nn.append('fsi_'+bd)
+             self.univeral_nn.append('fpi_'+bd)
+             self.univeral_nn.append('fpp_'+bd)
+             self.univeral_nn.append('fsi_'+bdr)
+             self.univeral_nn.append('fpi_'+bdr)
+             self.univeral_nn.append('fpp_'+bdr)
+
+      if not self.be_univeral_nn is None:
+         if self.be_univeral_nn=='all':
+            univeral_bonds = self.bonds
+         else:
+            univeral_bonds = self.be_univeral_nn
+         for bd in univeral_bonds:
+             b = bd.split('-')
+             bdr = b[1] + '-' + b[0]
+             self.univeral_nn.append('fe_'+bd)
+             self.univeral_nn.append('fe_'+bdr)
+             
+      if not self.mf_univeral_nn is None:
+         if self.mf_univeral_nn=='all':
+            univeral_bonds = self.spec
+         else:
+            univeral_bonds = self.mf_univeral_nn
+         for sp in self.spec:
+             for t in range(1,self.messages+1):
+                 self.univeral_nn.append('f'+str(t)+'_'+sp)
 
   def write_lib(self,libfile='ffield',loss=None):
       p_   = self.sess.run(self.p)
