@@ -139,7 +139,7 @@ class ReaxFF(object):
                popSize=500,
                fromPop=False,
                board=False,
-               penalty=100000.0,
+               lambda_bd=100000.0,
                to_train=True,
                optMethod='ADAM',
                maxstep=60000,
@@ -209,14 +209,14 @@ class ReaxFF(object):
          self.p_,self.m_ = self.ic.check(self.p_,self.m_,resetDeadNeuron=self.resetDeadNeuron)
 
       self.set_neurons()
-      self.penalty       = penalty
+      self.lambda_bd     = lambda_bd
       self.logger        = logger('training.log')
       self.initialized   = False  
 
       if self.VariablesToOpt is None:
          self.set_parameters(self.opt) 
       else:
-         self.set_parameters_ud() 
+         self.set_parameters_to_opt() 
 
 
   def init_bonds(self):
@@ -551,7 +551,7 @@ class ReaxFF(object):
              
              l_atol = tf.add(self.diffe[bd],l_atol)
              l_atol = tf.add(tf.nn.relu(self.rc_bo[bd]-self.rcut[bd]),l_atol)
-      return l_atol*self.penalty
+      return l_atol*self.lambda_bd
 
 
   def get_bond_energy(self):
@@ -1226,7 +1226,8 @@ class ReaxFF(object):
 
       if self.optword.find('notor')>=0:
          cons = cons + self.tor_v
-      self.ang_v = ['val1','val2','val3','val4','val5','val6','val7',
+      self.ang_v = ['theta0',
+                    'val1','val2','val3','val4','val5','val6','val7',
                     'pen1','pen2','pen3','pen4',
                     'coa1','coa2','coa3','coa4'] 
       if self.optword.find('noang')>=0:
@@ -1276,11 +1277,11 @@ class ReaxFF(object):
           elif key=='atomic':
              self.p[k] = tf.clip_by_value(self.v[k],-99.9999,999.9999)
           elif key=='acut':
-             self.p[k] = tf.clip_by_value(self.v[k],0.0001,0.0002)
+             self.p[k] = tf.clip_by_value(self.v[k],0.0000999,0.0002)
           elif key=='hbtol':
              self.p[k] = tf.clip_by_value(self.v[k],0.0001,0.0002)
           elif key=='cutoff':
-             self.p[k] = tf.clip_by_value(self.v[k],0.01,0.012)
+             self.p[k] = tf.clip_by_value(self.v[k],0.009999,0.012)
           elif key=='ovun5':
              self.p[k] = tf.clip_by_value(self.v[k],0.0,60.0*self.unit)
           elif key=='ovun1':
@@ -1290,17 +1291,17 @@ class ReaxFF(object):
           elif key == 'boc3':
              self.p[k] = tf.clip_by_value(self.v[k],0.0,50.0)
           elif key == 'gammaw':
-             self.p[k] = tf.clip_by_value(self.v[k],1.00,49.0)
+             self.p[k] = tf.clip_by_value(self.v[k],0.5,19.0)
           elif key == 'gamma':
              self.p[k] = tf.clip_by_value(self.v[k],0.001,6.0)
           elif key == 'pen1':
              self.p[k] = tf.clip_by_value(self.v[k],-69.0*self.unit,199.0*self.unit)
           elif key == 'Devdw':
-             self.p[k] = tf.clip_by_value(self.v[k],0.01*self.unit,9.0*self.unit)
+             self.p[k] = tf.clip_by_value(self.v[k],0.001*self.unit,0.333*self.unit)
           elif key == 'Dehb':
              self.p[k] = tf.clip_by_value(self.v[k],-3.0*self.unit,3.0*self.unit)
           elif key == 'Desi':
-             self.p[k] = tf.clip_by_value(self.v[k],100.000*self.unit,699.0*self.unit)
+             self.p[k] = tf.clip_by_value(self.v[k],10.0*self.unit,1000.0*self.unit)
           elif key== 'Depi':
              self.p[k] = tf.clip_by_value(self.v[k],0.000*self.unit,299.0*self.unit)
           elif key== 'Depp':
@@ -1372,7 +1373,7 @@ class ReaxFF(object):
              self.p[k] = tf.clip_by_value(self.v[k],1.5,3.6)
           elif key == 'rvdw':
              rvdw_ = 1.3*self.p_['rosi_'+vn[1]]
-             self.p[k] = tf.clip_by_value(self.v[k],rvdw_,2.0*self.p_['rosi_'+vn[1]])
+             self.p[k] = tf.clip_by_value(self.v[k],rvdw_,2.5*self.p_['rosi_'+vn[1]])
           elif key=='val5':
              self.p[k] = tf.clip_by_value(self.v[k],0.0,3.0)
           elif key=='val1':
@@ -1393,7 +1394,7 @@ class ReaxFF(object):
              self.p[k] = self.v[k]
 
 
-  def set_parameters_ud(self,opt=[],libfile=None):
+  def set_parameters_to_opt(self,libfile=None):
       if not libfile is None:
          self.p_,zpe,spec,bonds,offd,angs,torp,hbs = read_lib(libfile=libfile)
 
@@ -1662,7 +1663,7 @@ class ReaxFF(object):
       if self.VariablesToOpt is None:
          self.set_parameters(opt=opt,libfile=libfile)
       else:
-         self.set_parameters_ud()
+         self.set_parameters_to_opt()
 
       self.memory()
       self.set_zpe()
