@@ -22,15 +22,12 @@ def train(direcs=None,step=5000,batch=None,convergence=0.97,lossConvergence=1000
                 'ovun3':-2.0,'ovun4':-2.0,
                 'lp1':-3.0,'vdw1':-3.0},
           spec=[],
-          writelib=1000,nn=True,vdwnn=False,mefac=0.1,
-          bo_layer=[3,1],
-          mf_layer=[9,2],
-          be_layer=[3,1],
+          writelib=1000,nn=False,mefac=0.1,
           EnergyFunction=3,MessageFunction=2,mpopt=3,
           bore={'others':0.45},
           bom={'others':1.20},
           weight={'others':10.0},
-          penalty=10000.0,
+          lambda_bd=10000.0,
           ffield = 'ffield.json'):
     ''' training the force field '''
     rn = ReaxFF(libfile=ffield,
@@ -43,7 +40,7 @@ def train(direcs=None,step=5000,batch=None,convergence=0.97,lossConvergence=1000
                 pkl=False,
                 batch_size=batch,
                 losFunc='n2',
-                penalty=10000.0,
+                lambda_bd=lambda_bd,
                 nanv=nanv,
                 weight=weight,
                 bore=bore,
@@ -87,16 +84,16 @@ def train_mpnn(direcs=None,step=5000,batch=None,convergence=0.97,lossConvergence
                optword='nocoul',
                spec=[],
                writelib=1000,nn=True,vdwnn=False,
-               bo_layer=[3,1],
+               bo_layer=[4,1],
                mf_layer=[9,2],
-               be_layer=[3,1],
+               be_layer=[6,1],
                EnergyFunction=3,MessageFunction=2,mpopt=3,
                bore={'others':0.45},spv_be=False,
                bom={'others':1.0},spv_bm=False,
                mefac=0.1,
                weight={'others':10.0},
-               regularize=True,lambda_pen=0.0001,
-               penalty=100.0,
+               regularize=True,lambda_reg=0.0001,
+               lambda_bd=1000.0,
                ffield = 'ffield.json'):
     ''' train the massage passing model '''
     opt_ = None
@@ -125,8 +122,7 @@ def train_mpnn(direcs=None,step=5000,batch=None,convergence=0.97,lossConvergence
                 pkl=False,
                 batch_size=batch,
                 losFunc='n2',
-                penalty=penalty,
-                regularize=regularize,lambda_pen=lambda_pen,
+                regularize=regularize,lambda_reg=lambda_reg,lambda_bd=lambda_bd,
                 nanv=nanv,
                 bom=bom,bore=bore,spv_bm=spv_bm,spv_be=spv_be,
                 weight=weight,mefac=mefac,
@@ -139,7 +135,6 @@ def train_mpnn(direcs=None,step=5000,batch=None,convergence=0.97,lossConvergence
                                    writelib=writelib) 
 
     libstep = int(i - i%writelib)
-
     if i==libstep:
        libstep = libstep - writelib
     if libstep<=0:
